#! /usr/bin/bash

minikube start > minikube.log || echo "failure" >&2

eval $(minikube docker-env)

( cd python:simpleserver; docker build -t python:simpleserver . > /dev/null )
( cd python:newsimpleserver; docker build -t python:newsimpleserver . > /dev/null )

function pause() {
    echo -n "Press Enter to continue..."
    read
    echo
}

echo "Deploy pod from a template"
echo -e "\tjust displaying the template here"
echo "-------------------------------------------------------------------------"
pause

vim simpleserver.yml

echo "-------------------------------------------------------------------------"
echo

echo "Deploy the pod from its template"
echo "-------------------------------------------------------------------------"
echo "$ kubectl create -f simpleserver.yml"

kubectl create -f simpleserver.yml \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "List pods"
echo "-------------------------------------------------------------------------"
echo "$ kubectl get pods"

kubectl get pods

echo "-------------------------------------------------------------------------"
echo

pause

echo "Describe pod"
echo "-------------------------------------------------------------------------"
echo "$ kubectl describe pod/simpleserver"

kubectl describe pod/simpleserver

echo "-------------------------------------------------------------------------"
echo

pause

echo "Expose pod's port as a service"
echo "-------------------------------------------------------------------------"
echo "$ kubectl expose pod simpleserver --name=simpleservice --type=NodePort"

kubectl expose pod simpleserver --name=simpleservice --type=NodePort \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

url=$(minikube service simpleservice --url)

pause

echo "Test service from the pod"
echo "-------------------------------------------------------------------------"
echo "$ curl $url"

curl $url

echo "-------------------------------------------------------------------------"
echo

pause

kubectl delete service/simpleservice > /dev/null
kubectl delete pod/simpleserver > /dev/null

echo "Deploy application as a deployment from a template"
echo -e "\tjust displaying the template here"
echo "-------------------------------------------------------------------------"
pause

vim deployment.yml

echo "-------------------------------------------------------------------------"
echo

echo "Deploy the application from the deployment template"
echo "-------------------------------------------------------------------------"
echo "$ kubectl apply -f deployment.yml"

kubectl apply -f deployment.yml \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "List deployments"
echo "-------------------------------------------------------------------------"
echo "$ kubectl get deployments"

kubectl get deployments

echo "-------------------------------------------------------------------------"
echo

pause

echo "List replica sets"
echo "-------------------------------------------------------------------------"
echo "$ kubectl get replicasets"

kubectl get replicasets

echo "-------------------------------------------------------------------------"
echo

pause

echo "Expose deployment as a service"
echo -e "\thandles load-balancing and routing to actual pods"
echo "-------------------------------------------------------------------------"
echo "$ kubectl expose deployment/simpleserver --name=simpleservice"

kubectl expose deployment/simpleserver --name=simpleservice --type=NodePort \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

url=$(minikube service simpleservice --url)

pause

echo "Test service from the deployment"
echo "-------------------------------------------------------------------------"
echo "$ curl $url"

curl $url \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "Rollout new Docker image version"
echo "-------------------------------------------------------------------------"
echo "$ kubectl set image deployment/simpleserver pythonserver=python:newsimpleserver"

kubectl set image deployment/simpleserver pythonserver=python:newsimpleserver \
    || echo "failure" >&2

echo "-------------------------------------------------------------------------"
echo

pause

echo "List replica sets"
echo "-------------------------------------------------------------------------"
echo "$ kubectl get replicasets"

kubectl get replicasets

echo "-------------------------------------------------------------------------"
echo

pause

echo "List pods"
echo "-------------------------------------------------------------------------"
echo "$ kubectl get pods"

kubectl get pods

echo "-------------------------------------------------------------------------"
echo

echo "Test service from the deployment with the new application version"
echo "-------------------------------------------------------------------------"
echo "$ curl $url"

watch -n 5 curl $url

echo "-------------------------------------------------------------------------"
echo

pause

kubectl delete service/simpleservice
kubectl delete deployment/simpleserver

minikube delete --all >> minikube.log || echo "failure" >&2

